/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 03-04-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package pl.edu.icm.unicore.uvos.db;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.IntegerTypeHandler;
import org.apache.ibatis.type.JdbcType;


/**
 * Extends standard MyBatis {@link IntegerTypeHandler} so -1 int parameters are
 * mapped to a SQL value "NULL". Also NULL results are mapped back to -1.
 * 
 * @author K. Benedyczak
 */
public class NullIntTypeHandler extends BaseTypeHandler<Integer> {
	@Override
	public void setNonNullParameter(PreparedStatement ps, int i, Integer parameter, JdbcType jdbcType)
			throws SQLException {
		if (parameter == -1)
			ps.setNull(i, Types.INTEGER);
		else
			ps.setInt(i, parameter);
	}

	@Override
	public Integer getResult(ResultSet rs, int columnName)
			throws SQLException 
	{
		Integer result = getNullableResult(rs, columnName);
		if (rs.wasNull()) {
			return Integer.valueOf(-1);
		} else {
			return result;
		}
	}
	
	@Override
	public Integer getResult(ResultSet rs, String columnName)
			throws SQLException 
	{
		Integer result = getNullableResult(rs, columnName);
		if (rs.wasNull()) {
			return Integer.valueOf(-1);
		} else {
			return result;
		}
	}

	@Override
	public Integer getResult(CallableStatement cs, int columnIndex)
			throws SQLException 
	{
		Integer result = getNullableResult(cs, columnIndex);
		if (cs.wasNull()) {
			return Integer.valueOf(-1);
		} else {
			return result;
		}
	}

	@Override
	public Integer getNullableResult(ResultSet rs, String columnName)
			throws SQLException {
		return rs.getObject(columnName) == null ? -1 : rs.getInt(columnName);
	}

	@Override
	public Integer getNullableResult(ResultSet rs, int columnIndex)
			throws SQLException {
		return rs.getObject(columnIndex) == null ? -1 : rs.getInt(columnIndex);
	}

	@Override
	public Integer getNullableResult(CallableStatement cs, int columnIndex)
			throws SQLException {
		return cs.getObject(columnIndex) == null ? -1 : cs.getInt(columnIndex);
	}
}
