/*
 * Copyright (c) 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2008-08-08
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.mail;

import java.util.HashMap;
import java.util.Map;

import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.server.idresolver.Authenticator;


/**
 * Stores dynamic information used to fill notification message.
 * @author golbi
 */
public class VariablesMap
{
	private Map<String, String> map;
	private String action;
	
	public static final String VAR_ACTION = "${ACTION}";  
	public static final String VAR_CALLER = "${CALLER}";  

	public static final String VAR_SUBJECT = "${SUBJECT}";  
	public static final String VAR_GROUP = "${GROUP}";  
	
	public static final String VAR_TARGET = "${TARGET}";
	public static final String VAR_MOVED = "${MOVED}";
	public static final String VAR_EQUIVALENT = "${EQUIVALENT}";
	public static final String VAR_ATTRIBUTE = "${ATTRIBUTE}";
	public static final String VAR_STATUS = "${STATUS}";
	public static final String VAR_DATE = "${DATE}";
	
	public static final String VAR_APP_ID = "${APP_ID}";  
	public static final String VAR_FORM_NAME = "${FORM_NAME}";
	public static final String VAR_FORM_ID = "${FORM_ID}";
	public static final String VAR_FORM_GROUP = "${FORM_GROUP}";
	public static final String VAR_APP_ADMIN_NOTES = "${APP_ADMIN_NOTES}";
	public static final String VAR_APP_STATUS = "${APP_STATUS}";
	
	public VariablesMap(String action)
	{
		String caller;
		try
		{
			caller = Authenticator.getCurrentContext().getCaller().prettyString();
		} catch (InternalException e)
		{
			caller = "UNKNOWN";
		}
		map = new HashMap<String, String>();
		map.put(VAR_ACTION, action);
		map.put(VAR_CALLER, caller);
		map.put(VAR_SUBJECT, "-");
		map.put(VAR_GROUP, "-");
		map.put(VAR_ATTRIBUTE, "-");
		this.action = action;
	}
	
	public VariablesMap setSubject(String value)
	{
		map.put(VAR_SUBJECT, value);
		return this;
	}

	public VariablesMap setGroup(String value)
	{
		map.put(VAR_GROUP, value);
		return this;
	}

	public VariablesMap setAttriubute(String value)
	{
		map.put(VAR_ATTRIBUTE, value);
		return this;
	}
	
	public VariablesMap set(String key, String value)
	{
		map.put(key, value);
		return this;
	}
	
	Map<String, String> getMap()
	{
		return map;
	}
	
	String getGroup()
	{
		return map.get(VAR_GROUP);
	}
	
	String getAction()
	{
		return action;
	}
}