/*
 * Copyright (c) 2012 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE.txt file for licensing information.
 */
package pl.edu.icm.unicore.uvos.registration;

import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

import eu.emi.emir.client.EMIRClient;
import eu.unicore.util.Log;

import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;
import pl.edu.icm.unicore.uvos.util.GLUEInfoProvider;
import pl.edu.icm.unicore.uvos.util.ThreadScheduler;

/**
 * Handles EMIR (re)registrations.
 * @author K. Benedyczak
 */
public class EMIRHandler
{
	private static final Logger logger = Log.getLogger(UVOSServerProperties.LOG_PFX, EMIRHandler.class);
	private ScheduledExecutorService scheduler;
	private EMIRClient client;
	private GLUEInfoProvider infoProvider;
	
	public EMIRHandler(UVOSServerProperties conf, ThreadScheduler thScheduler, GLUEInfoProvider infoProvider)
	{
		scheduler = thScheduler.getScheduledExecutor();
		this.infoProvider = infoProvider;
		String emirUrl = conf.getValue(UVOSServerProperties.EMIR_REGISTRY);
		if (emirUrl != null)
			client = new EMIRClient(emirUrl, conf.getClientConfiguration());
	}
	
	public void start()
	{
		if (client == null) 
		{
			logger.debug("EMIR subsystem won't be started as no EMIR address is defined");
			return;
		}
		Adder adder = new Adder();
		scheduler.schedule(adder, 3, TimeUnit.SECONDS);
	}
	
	public class Adder implements Runnable
	{
		public void run()
		{
			UVOSEMIRClient emirClient = new UVOSEMIRClient(client, infoProvider);
			Date next = emirClient.readd();
			Calendar now = Calendar.getInstance();
			long wait = (next.getTime() - now.getTimeInMillis())/2;
			logger.debug("Delay for registry update is: " + wait);
			scheduler.schedule(this, wait, TimeUnit.MILLISECONDS);
		}
	}
}
