/*
 * Copyright (c) 2012 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE.txt file for licensing information.
 */
package pl.edu.icm.unicore.uvos.registration;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import javax.ws.rs.core.MediaType;

import org.apache.log4j.Logger;
import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;

import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;
import pl.edu.icm.unicore.uvos.util.GLUEInfoProvider;

import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.ClientResponse.Status;

import eu.emi.emir.client.EMIRClient;
import eu.emi.emir.client.ServiceBasicAttributeNames;
import eu.unicore.util.Log;

public class UVOSEMIRClient
{
	private static final Logger logger = Log.getLogger(UVOSServerProperties.LOG_PFX, EMIRClient.class);
	private EMIRClient client;
	private GLUEInfoProvider infoProvider;
	
	public UVOSEMIRClient(EMIRClient client, GLUEInfoProvider infoProvider)
	{
		this.client = client;
		this.infoProvider = infoProvider;
	}


	public Date readd()
	{
		try
		{
			JSONArray desc = infoProvider.getEndpointDescription();
			WebResource serviceAdmin = client.getClientResource().path("/serviceadmin");
			ClientResponse res = serviceAdmin.accept(MediaType.APPLICATION_JSON_TYPE).put(
							ClientResponse.class, desc);
			if (res.getClientResponseStatus() != Status.OK)
				throw new Exception("Unable to register service to EMIR: " + 
						res.getClientResponseStatus());

			logger.info("Successfully registered service at EMI registry " + 
					client.getClientResource().getURI());
			JSONArray ret = res.getEntity(JSONArray.class);
			if (ret.length() != 1)
				throw new Exception("EMIR returned response array of != 1 size: " + ret.toString());
			
			JSONObject registered = ret.getJSONObject(0);
			JSONObject expireOn = registered.getJSONObject(
					ServiceBasicAttributeNames.SERVICE_EXPIRE_ON.toString());
			return getDate(expireOn);
			
		} catch (Exception e)
		{
			logger.error("Can't add service to EMI registry " + 
					client.getClientResource().getURI(), e);
			Calendar minTT = Calendar.getInstance();
			minTT.add(Calendar.MINUTE, 10);
			return new Date(minTT.getTimeInMillis());
		}
	}
	
	
	private Date getDate(JSONObject json) throws JSONException, ParseException {
		String strDate = json.getString("$date");
		SimpleDateFormat formatter = new SimpleDateFormat(
			"yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
	 	formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
	 	return formatter.parse(strDate);
	}
}
