package pl.edu.icm.unicore.uvos.saml;

import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;

/**
 *
 * @author K. Stasiak
 */
public class SAMLConfiguration {
	private boolean signRespNever;
	private boolean signRespAlways;
	private boolean signIfRespUnsigned;
	private boolean signAssertNever;
	private boolean signAssertAlways;
	public enum ResponseSigningPolicy {always, never, asRequest};
	public enum AssertionSigningPolicy {always, never, ifResponseUnsigned, asRequest};

	private void init(UVOSServerProperties conf)
	{
		ResponseSigningPolicy repPolicy = conf.getEnumValue(UVOSServerProperties.SIGN_RESPONSE, ResponseSigningPolicy.class);
		signRespAlways = signRespNever = false;
		if (repPolicy == ResponseSigningPolicy.always)
			signRespAlways = true;
		else if (repPolicy == ResponseSigningPolicy.never)
			signRespNever = true;

		AssertionSigningPolicy assertionPolicy = conf.getEnumValue(UVOSServerProperties.SIGN_ASSERTION, AssertionSigningPolicy.class);
		signAssertAlways = signAssertNever = signIfRespUnsigned = false;
		if (assertionPolicy == AssertionSigningPolicy.always)
			signAssertAlways = true;
		else if (assertionPolicy == AssertionSigningPolicy.never)
			signAssertNever = true;
		else if (assertionPolicy == AssertionSigningPolicy.ifResponseUnsigned)
			signIfRespUnsigned = true;
	}

	public SAMLConfiguration(UVOSServerProperties conf){
		init(conf);
	}

	public boolean isSignRespNever() {
		return signRespNever;
	}

	public boolean isSignRespAlways() {
		return signRespAlways;
	}

	public boolean isSignIfRespUnsigned() {
		return signIfRespUnsigned;
	}

	public boolean isSignAssertNever() {
		return signAssertNever;
	}

	public boolean isSignAssertAlways() {
		return signAssertAlways;
	}
}
