/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 21, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.saml;

import java.security.cert.X509Certificate;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.elements.NameID;
import eu.unicore.samly2.elements.NameIDPolicy;
import eu.unicore.samly2.exceptions.SAMLProtocolException;
import eu.unicore.samly2.proto.NameIDMappingRequest;
import eu.unicore.samly2.proto.NameIDMappingResponse;
import eu.unicore.security.dsig.DSigException;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.server.IssuerIdentityProvider;
import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import xmlbeans.org.oasis.saml2.protocol.NameIDMappingRequestDocument;

/**
 * Utility for handling request->response chain.
 * @author K. Benedyczak
 */
public class SAMLNameMappingUtil extends SAMLUtil
{
	private NameIDMappingRequestDocument requestDoc;
	private NameIDMappingRequest req;
	
	private String requestedFormat;
	private Identity originalIdentity;
	
	public SAMLNameMappingUtil(NameIDMappingRequestDocument request, 
			IssuerIdentityProvider secProvider, UVOSServerProperties conf, SAMLConfiguration samlConfig)
	{
		super(request.getNameIDMappingRequest(), secProvider, conf, samlConfig);
		this.requestDoc = request;
	}
	
	public void parse() throws SAMLProtocolException
	{
		req = new NameIDMappingRequest(requestDoc);
		NameIDPolicy policy = req.getPolicy();
		requestedFormat = policy.getXBean().getFormat();
		if (requestedFormat == null)
			requestedFormat = SAMLConstants.NFORMAT_UNSPEC;
		originalIdentity = mapNameID(req.getNameToMap().getXBean());
	}
	
	public Identity getOriginalIdentity()
	{
		return originalIdentity;
	}

	public String getRequestedFormat()
	{
		return requestedFormat;
	}
	
	public NameIDMappingResponse getOKResponse(NameID mapped)
	{
		return new NameIDMappingResponse(getIssuer(), req.getID(), mapped);
	}
	
	public NameIDMappingResponse getErrorResponse(SAMLProtocolException e)
	{
		return new NameIDMappingResponse(getIssuer(), req.getID(), e);
	}

	public NameIDMappingResponse getErrorResponse(UVOSException e, String message) 
		throws UVOSFault
	{
		return getErrorResponse(convert2SAMLError(e, message, false));
	}	
	
	public NameIDMappingResponse getErrorResponse(UVOSException e) 
		throws UVOSFault
	{
		return getErrorResponse(convert2SAMLError(e, null, true));
	}

	@Override
	public X509Certificate[] getIssuerIdentityFromSignature() 
		throws DSigException
	{
		return getIssuerIdentityFromSignature(req);
	}	

	public boolean doSignResponse()
	{
		if (samlConfiguration.isSignRespAlways())
			return true;
		if (samlConfiguration.isSignRespNever())
			return false;
		return req.isSigned();
	}
}
