/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Nov 30, 2009
 * Author: T. Rekawek <newton@mat.uni.torun.pl>
 */

package pl.edu.icm.unicore.uvos.server;

import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownHostException;

import org.apache.log4j.Logger;

import eu.unicore.util.Log;

import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;

/**
 * Implementation of CryptoDataProvider
 * @author K. Benedyczak
 */
public class IssuerIdentityProviderImpl implements IssuerIdentityProvider
{
	private static final Logger log = Log.getLogger(UVOSServerProperties.LOG_PFX, IssuerIdentityProviderImpl.class);
	private String issuerURI;

	public IssuerIdentityProviderImpl(UVOSServerProperties config) throws Exception
	{
		issuerURI = establishServerURI(config);
	}
	
	public String getIssuerURI()
	{
		return issuerURI;
	}

	/**
	 * Tries to establish an URL which will identify this server. It will be used
	 * (if not overriden by config option) in Issuer element of SAML Respones.
	 * <p>
	 * Warning: it is not guaranteed to succeed - it is much safer to set the 
	 * URI by hand in config file.
	 */
	private String establishServerURI(UVOSServerProperties config)
	{
		String uri = config.getValue(UVOSServerProperties.ISSUER_URI);
		if (uri == null)
		{
			log.info("No " + UVOSServerProperties.P + UVOSServerProperties.ISSUER_URI + " is set. " +
					"Will try to guess it." );
		} else
		{
			try
			{
				new URI(uri);
				return uri;
			} catch (URISyntaxException e)
			{
				log.info(UVOSServerProperties.ISSUER_URI + 
					" is set but is invalid (isn't a URI): " +
					e.getMessage() + ". Will try to guess it." );
			}
		}
		
		String fqdn;
		boolean ssl = config.getBooleanValue(UVOSServerProperties.HTTPS_ENABLE);
		try
		{
			InetAddress addr = InetAddress.getLocalHost();
			fqdn = addr.getCanonicalHostName();
		} catch (UnknownHostException e)
		{
			log.warn("Can't determine host domain name " +
				"and the issuer URI isn't specified. " +
				"Will do the best to produce the the URL but" +
				"it may be incorrect.");
			if (ssl)
				fqdn = config.getValue(UVOSServerProperties.HTTPS_HOST);
			else
				fqdn = config.getValue(UVOSServerProperties.HTTP_HOST);
		}
		
		int port = -1;
		if (ssl)
			port = config.getIntValue(UVOSServerProperties.HTTPS_PORT);
		else
			port =	config.getIntValue(UVOSServerProperties.HTTP_PORT);
		uri = "http://" + fqdn + ":" + port; 
		log.info("Will use autogenerated server's URI: " + uri);
		return uri;  
	}
}
