/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 8, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.server;

import java.io.File;
import java.net.URL;

import org.codehaus.xfire.transport.http.XFireServlet;
import org.eclipse.jetty.deploy.DeploymentManager;
import org.eclipse.jetty.deploy.providers.WebAppProvider;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import org.eclipse.jetty.server.handler.DefaultHandler;
import org.eclipse.jetty.server.handler.HandlerCollection;
import org.eclipse.jetty.server.handler.RequestLogHandler;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.picocontainer.PicoContainer;

import eu.unicore.util.configuration.ConfigurationException;
import eu.unicore.util.jetty.JettyServerBase;

import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;

/**
 * Jetty server implementation. Here appropriate connectors are configured.
 * 
 * This module is not managed by Pico container as it depends on it. The dependency is required
 * to setup web applications (which may not use Pico at all, and e.g. Spring container).
 * 
 * @author K. Benedyczak
 */
public class JettyServer extends JettyServerBase
{
	public static final String UVOS_SERVER_CONTAINER_REFERENCE = "uvos.container.ref";
	
	private String webappsDir;
	private String webappsTmpDir;
	private PicoContainer picoReference;

	public JettyServer(UVOSServerProperties conf, final PicoContainer pico)
			throws Exception
	{
		super(createURLs(conf), conf.getAuthAndTrust(), conf.getJettyProperties(), null);
		this.picoReference = pico;
		webappsDir = conf.getValue(UVOSServerProperties.WEBAPPS_DIR);
		webappsTmpDir = System.getProperty("java.io.tmpdir") + File.separator + "uvos-webappstmp";
		File f = new File(webappsTmpDir);
		if (!f.exists())
			f.mkdir();
		initServer();
	}
	
	private static URL[] createURLs(UVOSServerProperties conf) throws Exception
	{
		boolean plain = conf.getBooleanValue(UVOSServerProperties.HTTP_ENABLE);
		boolean ssl = conf.getBooleanValue(UVOSServerProperties.HTTPS_ENABLE);
		if (!(plain || ssl))
			throw new Exception("No one connector is enabled");
		
		URL []ret;
		if (plain && ssl)
			ret = new URL[2];
		else
			ret = new URL[1];
		int i=0;
		
		if (plain)
			ret[i++] = new URL("http://" + conf.getValue(UVOSServerProperties.HTTP_HOST) + 
					":" + conf.getValue(UVOSServerProperties.HTTP_PORT));
		if (ssl)
			ret[i++] = new URL("https://" + conf.getValue(UVOSServerProperties.HTTPS_HOST) + 
					":" + conf.getValue(UVOSServerProperties.HTTPS_PORT));
		return ret;
	}
	
	@Override
	protected Handler createRootHandler() throws ConfigurationException
	{
		ServletContextHandler root=new ServletContextHandler(getServer(), "/", ServletContextHandler.SESSIONS);
		root.addServlet(XFireServlet.class, "/*");

		final ContextHandlerCollection contexts = new ContextHandlerCollection();
		RequestLogHandler requestLogHandler = new RequestLogHandler();

		HandlerCollection handlers = new HandlerCollection();
		handlers.setHandlers(new Handler[]{
				contexts,
				root,
				new DefaultHandler(),
				requestLogHandler});
		WebAppProvider provider = new WebAppProvider();
		provider.setParentLoaderPriority(false);
		provider.setExtractWars(true);
		provider.setMonitoredDirName(webappsDir);
		provider.setTempDir(new File(webappsTmpDir));
		provider.setScanInterval(10);
	
		DeploymentManager deployManager = new DeploymentManager();
		deployManager.addAppProvider(provider);
		
		deployManager.setContexts(contexts);
		deployManager.setContextAttribute(UVOS_SERVER_CONTAINER_REFERENCE, 
				picoReference);
		getServer().addBean(deployManager);
		
		return handlers;
	}
}
