/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 17, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.server.idresolver;

import java.util.Map;

import eu.unicore.security.HTTPAuthNTokens;
import eu.unicore.security.SecurityTokens;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.InvalidValueException;
import pl.edu.icm.unicore.uvos.engine.InternalAuthorization;
import pl.edu.icm.unicore.uvos.engine.SecurityContext;

/**
 * Resolves HTTP login&passwd into email type {@link Identity}.
 *  
 * @author K. Benedyczak
 */
public class LoginResolver extends AbstractResolver
{
	public LoginResolver(InternalAuthorization voAuthz)
	{
		super(voAuthz);
	}

	public SecurityContext resolve(SecurityTokens input) 
		throws IdUnresolvedException, InternalException, InvalidValueException
	{
		Map<String, Object> secCtx = input.getContext();
		if (secCtx == null)
			return null;
		HTTPAuthNTokens fromHttp = (HTTPAuthNTokens) secCtx.get(
				SecurityTokens.CTX_LOGIN_HTTP);
		if (fromHttp == null)
			return null;
		
		Identity idRet = new Identity(IdentityType.EMAIL, 
				fromHttp.getUserName(), null);
		idRet.setAuthnData(fromHttp.getPasswd());
		
		return authenticate(idRet, "Email");
	}
}
