/*
 * Copyright (c) 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Jul 16, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.server.saml;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import eu.unicore.util.Log;

import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;

/**
 * Keeps track of the recent requests ids and informs if there is duplicate.
 * Makes sense only for signed requests.
 * @author golbi
 */
public class ReplayAttackChecker
{
	private static final Logger log = Log.getLogger(UVOSServerProperties.LOG_PFX, ReplayAttackChecker.class);
	
	private Map<String, Long> usedMap;
	private long holdingTime;
	private long lastCleanupTime;
	
	public ReplayAttackChecker(UVOSServerProperties conf)
	{
		holdingTime = conf.getIntValue(UVOSServerProperties.SAML_REQUEST_VALIDITY) * 10;
		usedMap = new HashMap<String, Long>();
		lastCleanupTime = System.currentTimeMillis();
		log.info("Request IDs will be kept for at least " + holdingTime + "s");
		holdingTime *= 1000;
	}
	
	public synchronized void checkAndAdd(String id, long ts) throws Exception
	{
		if (usedMap.containsKey(id))
			throw new Exception("The request id was already seen.");
		usedMap.put(id, ts);
		if (System.currentTimeMillis() - lastCleanupTime > holdingTime)
			doCleanup();
	}
	
	private void doCleanup()
	{
		Iterator<Entry<String, Long>> entries = usedMap.entrySet().iterator();
		long now = System.currentTimeMillis();
		int cleaned = 0;
		while(entries.hasNext())
		{
			Entry<String, Long> entry = entries.next();
			if (now - entry.getValue() > holdingTime)
			{
				entries.remove();
				cleaned++;
			}
		}
		lastCleanupTime = now;
		if (log.isDebugEnabled())
			log.debug("Cleaned up recent ids: " + cleaned + " cleaned, " + 
					usedMap.size() + " retained.");
	}	
}
