/*
 * Copyright (c) 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Jul 28, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.util;

import java.io.InputStreamReader;
import java.util.List;

import pl.edu.icm.schemaparser.LDAPAttributeType;
import pl.edu.icm.schemaparser.generated.LdapLexer;
import pl.edu.icm.schemaparser.generated.LdapParser;


/**
 * This class invokes a LDAP schema parser on a standard input. 
 * After parsing all attribute types (object classess are ignored) are 
 * printed to standard output in a format suitable for UVOS. UVOS server
 * can read the output (quickly) upon startup and add new attribute types 
 * to its DB.
 * 
 * @author golbi
 */
class LdapSchemaParser
{
	private static final String info = 
		"\nThis program invokes a LDAP schema parser on its " +
		"standard input. " +
		"After parsing the input all attribute types " +
		"(object classess are ignored)" +
		" are printed to standard output in a format suitable for UVOS." +
		" UVOS server can read the output (quickly) upon startup and add" +
		" new attribute types to its DB.";
	
	public static void main(String[] args)
	{
		if (args.length != 0)
		{
			System.out.println(info);
			System.exit(0);
		}
		try
		{
			LdapLexer lexer = new LdapLexer(new InputStreamReader(
					System.in));
			
			LdapParser parser = new LdapParser(lexer);
			List<LDAPAttributeType> parsed = parser.startRule();
			for (LDAPAttributeType at: parsed)
			{
				List<String> names = at.getNames();
				String atName, atSDesc, atDesc; 
				
				atName = names.get(0);
				if (names.size() > 1)
				{
					int longest = 1;
					if (names.get(0).length() > names.get(1).length())
						longest = 0;
					atName = names.get((longest + 1)%2);
					atSDesc = names.get(longest);
				} else
					atSDesc = at.getDesc();
				
				atDesc = at.getDesc() + 
					"\\nThis attribute is equivalent to LDAP one with OID: "
					+ at.getOid();
				
				System.out.println("AT\n" + 
						atName + "\n" + 
						atSDesc + "\n" +
						atDesc +"\n");
			}
		} catch (Exception e)
		{
			System.err.println("exception: " + e);
		}
	}
}
