/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2009-09-23
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import eu.unicore.util.Log;


/**
 * Monitors log configuration file for changes and update runtime logging
 * facility if needed.
 * @author golbi
 * @author schuller
 */
public class LogFileWatcher
{
	private static final Logger log = Log.getLogger(UVOSServerProperties.LOG_PFX, LogFileWatcher.class);
	
	public static void startLogConfigWatcher(ScheduledExecutorService timer)
	{
		final String logConfig = System.getProperty("log4j.configuration");
		if (logConfig==null)
		{
			log.warn("No logger configuration found");
			return;
		}
		try
		{
			Runnable r = new Runnable()
			{
				public void run()
				{
					log.info("LOG CONFIG MODIFIED, re-configuring.");
					PropertyConfigurator.configure(logConfig);
				}
			};
			File logProperties = logConfig.startsWith("file:/") ? 
				new File(new URI(logConfig)) : new File(logConfig);
			FileWatcher fw = new FileWatcher(logProperties, r);
			timer.scheduleWithFixedDelay(fw, 5, 5, TimeUnit.SECONDS);
		}catch (FileNotFoundException fex)
		{
			log.warn("Log configuration file <" + logConfig + "> not found.");
		} catch (URISyntaxException e)
		{
			log.warn("Location of log configuration is not a valid URI: '"+logConfig+"'");
		}
	}
}
