/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 8, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.util;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;

import eu.unicore.util.Log;
import eu.unicore.util.configuration.ConfigurationException;
import eu.unicore.util.configuration.DocumentationReferenceMeta;
import eu.unicore.util.configuration.DocumentationReferencePrefix;
import eu.unicore.util.configuration.PropertyMD;
import eu.unicore.util.jetty.HttpServerProperties;

/**
 * UVOS overrides defaults of {@link HttpServerProperties} in case of allowing of anonymous SSL clients 
 * and NIO is enabled by default.
 * and has a custom alias for one option. This class also warns if typical misconfigurations are detected.
 * @author K. Benedyczak
 */
public class UvosHttpServerProperties extends HttpServerProperties
{
	private static final Logger log = Log.getLogger(Log.CONFIGURATION, UvosHttpServerProperties.class);
	
	@DocumentationReferencePrefix
	public static final String PREFIX = UVOSServerProperties.P+HttpServerProperties.DEFAULT_PREFIX;
	
	@DocumentationReferenceMeta
	public final static Map<String, PropertyMD> defaults=new HashMap<String, PropertyMD>();
	
	static{
		defaults.putAll(HttpServerProperties.defaults);
		defaults.put(USE_NIO, new PropertyMD("true").
				setDescription("Controls whether the NIO connector be used. NIO is best suited under high-load, when lots of connections exist that are idle for long periods."));
		defaults.put(REQUIRE_CLIENT_AUTHN, new PropertyMD().setHidden());
		
	}
	public UvosHttpServerProperties(Properties source, UVOSServerProperties base) throws ConfigurationException
	{
		super(source, PREFIX, defaults);
		if (isSet(REQUIRE_CLIENT_AUTHN))
		{
			log.warn("Do not use " + getKeyDescription(REQUIRE_CLIENT_AUTHN) + 
					" setting, it will be ignored. Use " + 
					base.getKeyDescription(UVOSServerProperties.HTTPS_ANON));
		}
		if (!base.isSet(UVOSServerProperties.HTTPS_ANON))
			setProperty(REQUIRE_CLIENT_AUTHN, ""+!UVOSServerProperties.httpsAnon);
		else
			setProperty(REQUIRE_CLIENT_AUTHN, ""+!base.getBooleanValue(UVOSServerProperties.HTTPS_ANON));
	}
}
