/*
 * Copyright (c) 2010 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-07-14
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.tests.db;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import eu.unicore.bugsreporter.annotation.RegressionTest;

import static junit.framework.Assert.*;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.engine.VirtualSecurityContextImpl;
import pl.edu.icm.unicore.uvos.engine.api.UVOSManagementInterface;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.idresolver.Authenticator;
import pl.edu.icm.unicore.uvos.tests.functional.AbstractTestBase;

public class AttributeStorageTest extends AbstractTestBase
{
	@Test
	@RegressionTest(url="https://sourceforge.net/tracker/?func=detail&aid=3388065&group_id=102081&atid=633902")
	public void testAttributesLoader()
	{
		attributeLoader.load();
		
		db.shutdown();
		
		ComponentContainer container;
		try
		{
			container = new ComponentContainer("src/test/resources/uvosServer.conf");
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail(e.toString());
			return;
		}
		Authenticator.injectFakeContext(new VirtualSecurityContextImpl(Permissions.ALL_P));
		voMan = container.getComponent(UVOSManagementInterface.class);
		try
		{
			int num = voMan.getAttributeTypes().size();
			assertTrue("Got " + num + " ATs", num >= 5);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail(e.toString());
		}
		
	}
	
	@Test
	public void testValues()
	{
		try
		{
			Identity id = new Identity(IdentityType.EMAIL, "a@a.com", "double");
			Element idEl = new Element(null, id);
			List<String> vals = new ArrayList<String>();
			int N = 0xD800;
			char[] chars = new char[N];
			chars[0] = (char)1;
			for (char i=1; i<N; i++)
				chars[i] = i;
			vals.add(new String(chars));
			
			Attribute a = new Attribute("A", vals, null);
			voMan.addIdentity(id);
			voMan.setAttribute(idEl, a, false);
			List<Attribute> ret = voQuery.getAttributes(idEl, "A", false, false, false);
			assertTrue(ret.size() == 1);
			Attribute abis = ret.get(0);
			assertTrue(abis.equals(a));
		} catch (Exception e)
		{
			e.printStackTrace();
			fail(e.toString());
		}
	}
}
