/*
 * Copyright (c) 2010 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2011-04-27
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package pl.edu.icm.unicore.uvos.tests.db;

import org.junit.Assert;
import org.junit.Test;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.exceptions.ElementAlreadyExistsException;
import pl.edu.icm.unicore.uvos.tests.functional.AbstractTestBase;

/**
 * Tests if the following constraints are met:
 * IDENTITIES (CMP, TYPEID, REMOVED_ID)
 * 
 * This is as in case of MySQL DB implementation is non trivial (triggers)
 */
public class UniqueConstraintTest extends AbstractTestBase
{
	@Test
	public void testIdentity()
	{
		Identity id = new Identity(IdentityType.EMAIL, "a@a.com", "double");
		try
		{
			voMan.addIdentity(id);
		} catch (Exception e)
		{
			e.printStackTrace();
			Assert.fail(e.toString());
		}
		try
		{
			id.setLabel("newLabel");
			voMan.addIdentity(id);
			Assert.fail("Managed to add same identity twice");
		} catch (ElementAlreadyExistsException e) { //OK 
		} catch (Exception e)
		{
			e.printStackTrace();
			Assert.fail("Got wrong exception: " + e.toString());
		}
		try
		{
			voMan.removeIdentity(id);
		} catch (Exception e)
		{
			e.printStackTrace();
			Assert.fail(e.toString());
		}
		try
		{
			voMan.addIdentity(id);
		} catch (Exception e)
		{
			e.printStackTrace();
			Assert.fail("Failed to add identity as removed" + e.toString());
		}
	}
}
