/*
 * Copyright (c) 2011-2012 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE.txt file for licensing information.
 */
package pl.edu.icm.unicore.uvos.tests.db;

import static junit.framework.Assert.fail;

import org.apache.ibatis.session.SqlSession;
import org.junit.Before;
import org.junit.Test;

import eu.unicore.bugsreporter.annotation.RegressionTest;

import pl.edu.icm.unicore.uvos.db.DB;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;

public class WatchdogTest
{
	private DB db;
	
	@Before
	public void init()
	{
		try
		{
			InitDB.main(new String[]{"src/test/resources/uvosServer.conf", "-b"});
			ComponentContainer container = new ComponentContainer("src/test/resources/uvosServer.conf");
			db = container.getComponent(DB.class);
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
	}
	
	@Test
	@RegressionTest(url="https://sourceforge.net/tracker/index.php?func=detail&aid=3476056&group_id=102081&atid=633902")
	public void testWatchdog() throws InterruptedException
	{
		Thread[] threads = new Thread[10];
		threads[0] = new HangingThread();
		threads[0].start();
		for (int i=1; i<threads.length; i++)
		{
			threads[i] = new LaggingThread();
			threads[i].start();
			Thread.sleep(300);
		}
		for (int i=0; i<threads.length; i++)
			threads[i].join();
	}
	
	private class LaggingThread extends Thread
	{
		public void run()
		{
			try
			{
				SqlSession session = db.getSqlSession();
				Thread.sleep(3000);
				db.releaseSqlSession(session);
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}
	}

	private class HangingThread extends Thread
	{
		public void run()
		{
			setName("BadGuy!!");
			db.getSqlSession();
		}
	}
}
