/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 20, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.tests.functional;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import eu.unicore.bugsreporter.annotation.RegressionTest;

import static junit.framework.Assert.*;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.AttributeType;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.engine.SecurityContext;
import pl.edu.icm.unicore.uvos.server.idresolver.Authenticator;


/**
 * Performs functional tests. Assumptions:
 * Server is up and running and DB is empty
 * @author K. Benedyczak
 */
public class AttributeAuthzFilterTest extends AbstractTestBase
{
	private AttributeType atType;
	
	@RegressionTest(url="https://sourceforge.net/tracker/index.php?func=detail&aid=3163920&group_id=102081&atid=633902",
			description="tstAttrs() does the job")
	@Test
	public void testAll()
	{
		try
		{
			createTestGI();
			atType = voMan.getAttributeTypes().get(0);
			tstAttrs();
			cleanTestGI();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Exception: " + e.getMessage());
		}
	}

	/**
	 * Tests if attributes in groups where client has access are returned, when the query subject attribute 
	 * is defined in the group where there is no access (so only this attribute as implied in the above group should
	 * be returned).
	 * @throws Exception
	 */
	private void tstAttrs() throws Exception
	{
		voMan.addToGroup(idA1, gZ);
		voMan.addToGroup(idB, gC);
		List<String> vals = new ArrayList<String>();
		vals.add("A1");
		Attribute attr1 = new Attribute(atType, vals, "/A/Z");
		Element elidAingZ = new Element(gZ, idA1);
		Element elidA = new Element(null, idA1);
		
		voMan.setAttribute(elidAingZ, attr1, false);
		
		SecurityContext ctx2 = internalAuthz.establishSecurityContext(idB);
		Authenticator.injectFakeContext(ctx2);
		List<Attribute> al1 = voQuery.getAttributes(elidA, null, true, true, true);
		Authenticator.injectFakeContext(ctx);

		if (al1.size() == 0)
			fail("All attributes were filtered out inget-all query");
		if (al1.size() != 1)
			fail("Wrong attr result of ig after get-all query: " + al1.size());
	}
}