package pl.edu.icm.unicore.uvos.tests.functional;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.server.saml.AttributeFilterImpl;
import pl.edu.icm.unicore.uvos.util.ThreadScheduler;

/**
 * 
 * @author K. Stasiak
 */
public class AttributeFilterFunctionalTest
{	
	
	private void savePolicy(String contents, String path) throws IOException
	{
		File p2 = new File(path);
		if (p2.exists())
			p2.delete();
		BufferedOutputStream fos = new BufferedOutputStream(
				new FileOutputStream(path));
		fos.write(contents.getBytes());
		fos.flush();
		fos.close();
	}

	@Test
	public void testReload() throws IOException
	{
		savePolicy("", "target/test-policy.properties");
		File f = new File("target/test-policy.properties");
		long lastAccessed = f.lastModified();
		AttributeFilterImpl attrFilter = new AttributeFilterImpl(
				"target/test-policy.properties", 
				100, new ThreadScheduler());
		List<Attribute> attrs = Arrays.asList(
				new Attribute("name", Arrays.asList("ok"), "/"),
				new Attribute("uvos:name", Arrays.asList("ok"), "/"),
				new Attribute("name", Arrays.asList("ok"), "/public"),
				new Attribute("name", Arrays.asList("ok"), 
						"/public/something"));
		List<Attribute> fattrs = attrFilter.filter(attrs);
		check(attrs, fattrs);
		try
		{
			Thread.sleep(1000);
		} catch (InterruptedException e)
		{
			Assert.fail("Unexpected interrupt");
		}
		savePolicy("excludedAttribute.1=name\n", "target/test-policy.properties");
		Assert.assertTrue("Ups - file modification didn't occured??", 
				f.lastModified() > lastAccessed);
		try
		{
			Thread.sleep(500);
		} catch (InterruptedException e)
		{
			Assert.fail("Unexpected interrupt");
		}
		fattrs = attrFilter.filter(attrs);
		Assert.assertEquals("Policy was not reloaded", 
				1, fattrs.size());
	}
	
	@Test
	public void testEmpty() throws IOException
	{
		AttributeFilterImpl attrFilter = new AttributeFilterImpl(
				"src/test/resources/attributeFilters-empty.properties", 
				5000, new ThreadScheduler());
		List<Attribute> attrs = Arrays.asList(
				new Attribute("name", Arrays.asList("ok"), "/"),
				new Attribute("uvos:name", Arrays.asList("ok"), "/"),
				new Attribute("name", Arrays.asList("ok"), "/public"),
				new Attribute("name", Arrays.asList("ok"), 
						"/public/something"));
		List<Attribute> fattrs = attrFilter.filter(attrs);
		check(attrs, fattrs);
	}

	@Test
	public void testInclude() throws IOException
	{
		AttributeFilterImpl attrFilter = new AttributeFilterImpl(
				"src/test/resources/attributeFilters-include.properties", 
				5000, new ThreadScheduler());
		
		List<Attribute> attrs = Arrays.asList(
				new Attribute("name", Arrays.asList("wrong"), "/"),
				new Attribute("uvos:name", Arrays.asList("ok"), "/"),
				new Attribute("uvos:name", Arrays.asList("wrong"), "/secret"),
				new Attribute("name", Arrays.asList("ok"), "/public"),
				new Attribute("name", Arrays.asList("wrong"), 
						"/public/private"));
		List<Attribute> fattrs = attrFilter.filter(attrs);
		check(attrs, fattrs);
	}

	@Test
	public void testExclude() throws IOException
	{
		AttributeFilterImpl attrFilter = new AttributeFilterImpl(
				"src/test/resources/attributeFilters-exclude.properties", 
				5000, new ThreadScheduler());
		
		List<Attribute> attrs = Arrays.asList(
				new Attribute("somesecret", Arrays.asList("wrong"), "/"),
				new Attribute("secret", Arrays.asList("wrong"), "/"),
				new Attribute("weak", Arrays.asList("ok"), "/a"),
				new Attribute("weak", Arrays.asList("wrong"), "/a/b"),
				new Attribute("weak", Arrays.asList("ok"), "/public/b"),
				new Attribute("name", Arrays.asList("ok"), "/public"));
		List<Attribute> fattrs = attrFilter.filter(attrs);
		check(attrs, fattrs);
	}

	
	@Test
	public void testAll() throws IOException
	{
		AttributeFilterImpl attrFilter = new AttributeFilterImpl(
				"src/test/resources/attributeFilters.properties", 
				5000, new ThreadScheduler());
		List<Attribute> attrs = Arrays.asList(
				new Attribute("name", Arrays.asList("wrong"), "/"),
				new Attribute("uvos:name", Arrays.asList("ok"), "/"),
				new Attribute("name", Arrays.asList("ok"), "/public"),
				new Attribute("name", Arrays.asList("wrong"), 
						"/public/private"),
				new Attribute("uvos:name", Arrays.asList("wrong"), "/secret"),
				new Attribute("uvos:name", Arrays.asList("wrong"), "/secret2"),
				new Attribute("uvos:secret", Arrays.asList("wrong"), "/"),
				new Attribute("uvos:weak_secret", Arrays.asList("ok"), "/something"),
				new Attribute("uvos:weak_secret", Arrays.asList("wrong"),
						"/something/something"),
				new Attribute("uvos:weak_secret", Arrays.asList("ok"), "/public"),
				new Attribute("uvos:weak_secret", Arrays.asList("ok"),
						"/public/something"));
		List<Attribute> fattrs = attrFilter.filter(attrs);
		check(attrs, fattrs);
	}



	private void check(List<Attribute> attrs, List<Attribute> fattrs)
	{
		int expected_size = 0;

		for (Attribute a : attrs)
			if ("ok".equals(a.getValues().get(0)))
				expected_size++;
		boolean failed = false;
		for (Attribute a : fattrs)
		{
			if ("wrong".equals(a.getValues().get(0)))
			{
				System.out.println(a + " is a false positive");
				failed = true;
			}
		}
		if (expected_size != fattrs.size())
		{
			System.out.println("Attributes after filtering are: ");
			for (Attribute a: fattrs)
				System.out.println("  " + a);
			Assert.fail("Not all exposed atributes were exposed");
		}
		if (failed)
			Assert.fail("Got false positives");
	}
}
