/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 20, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.tests.functional;

import java.util.Date;
import java.util.List;

import org.junit.Test;

import eu.unicore.bugsreporter.annotation.RegressionTest;

import static junit.framework.Assert.*;

import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;

/**
 * Performs functional tests. Assumptions:
 * Server is up and running and DB is empty
 * @author K. Benedyczak
 */
public class CompositeFunctionalTest extends AbstractTestBase
{
	@RegressionTest(url="https://sourceforge.net/tracker/index.php?func=detail&aid=3163916&group_id=102081&atid=633902",
			description="tstAllGroups checks the former bug condition")
	@Test
	public void testAll()
	{
		try
		{
			createTestGI();
			tstAddToGroup();
			tstRemoveFromGroup();
			cleanTestGI();

			createTestGI();
			tstMembership();
			cleanTestGI();

			createTestGI();
			tstAllGroups();
			cleanTestGI();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Exception: " + e.getMessage());
		}
	}
	
	private void tstAllGroups() throws Exception
	{
		voMan.addToGroup(idA1, gB);
		voMan.addToGroup(idA1, gZ);
		List<Group> groups = voQuery.getAllGroups(idA1, true);
		assertTrue(groups.size() == 3);
		assertTrue(groups.contains(gB));
		assertTrue(groups.contains(gZ));
		assertTrue(groups.contains(gA));
	}
	
	private void tstAddToGroup() throws Exception
	{
		voMan.addToGroup(idA1, gC);
		voMan.addToGroup(idB, gA);
		List<Element> c = voQuery.getGroupContent(gC);
		if (c.size() != 2)
			fail("Group " + gC + " should have two elements, has " + c.size());
		Element e = c.get(0);
		if (e.getGroup() != null || 
			!(e.getIdentity().equals(idA1) || e.getIdentity().equals(idA2)))
			fail("Group " + gC + " should contain " + idA1 + 
					" or " + idA2 + " but contains " + e);
		c = voQuery.getGroupContent(gA);
		if (c.size() != 3)
			fail("Group " + gA + " should have three elements, has " + c.size());
		e = c.get(0);
		if (e.getIdentity() != null && !e.getIdentity().equals(idB))
			fail("Identity in group " + gA + " should be " + idB + 
					" but is " + e);
	}

	private void tstRemoveFromGroup() throws Exception
	{
		voMan.removeFromGroup(idA1, gC);
		voMan.removeFromGroup(idB, gA);
		List<Element> c = voQuery.getGroupContent(gC);
		if (c.size() != 0)
			fail("Group " + gC + " should have no elements, has " + c.size());
		c = voQuery.getGroupContent(gA);
		if (c.size() != 2)
			fail("Group " + gA + " should have two elements, has " + c.size());		
	}
	
	private void tstMembership() throws Exception
	{
		voMan.addToGroup(idA1, gB);
		voMan.addToGroup(idA1, gZ);
		Date s = new Date(); 
		List<Group> groups = voQuery.getAllGroups(idA1, true);
		assertTrue(groups.size() == 3);
		
		groups = voQuery.getAllGroups(idA1, false);
		assertTrue(groups.size() == 2);
		
		groups = voQueryHistory.getAllGroups(idA1, true, s);
		assertTrue(groups.size() == 3);
		
		groups = voQueryHistory.getAllGroups(idA1, false, s);
		assertTrue(groups.size() == 2);
	}
}














