/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 20, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.tests.functional;

import java.io.FileInputStream;
import java.util.Date;
import java.util.List;

import org.junit.Test;

import static junit.framework.Assert.*;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;

/**
 * Performs functional tests. Assumptions:
 * Server is up and running and DB is empty
 * @author K. Benedyczak
 */
public class IdentitiesFunctionalTest extends AbstractTestBase
{
	@Test
	public void testAll()
	{
		try
		{
			tstBasic();
			tstIdentities();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Exception: " + e.getMessage());
		}
	}
	
	private void tstBasic() throws Exception
	{
		voMan.getIdentityTypes();
		voMan.getAttributeTypes();
	}
	
	private void tstIdentities() throws Exception
	{
		FileInputStream fis = new FileInputStream("src/test/resources/certs/pubCert.pem");
		int len = fis.available();
		byte []buffer = new byte[len];
		fis.read(buffer);
		Identity []idA = new Identity[7];
		boolean []comIds = {false, false, false, false, false, false, false};
		idA[0] = new Identity(IdentityType.DN, "cn=ala,c=pl,ou=icm", null);
		idA[1] = new Identity(IdentityType.X509CERT,  new String(buffer), null);
		idA[2] = new Identity(IdentityType.EMAIL, "a@b", null);
		idA[3] = new Identity(IdentityType.EMAIL, "bll@aa", "someLabel");
		idA[4] = new Identity(IdentityType.EMAIL, "2@a", null);
		idA[5] = new Identity(IdentityType.EMAIL, "3@a", null);
		idA[6] = new Identity(IdentityType.EMAIL, "4@a", null);
		voMan.addIdentity(idA[0]);
		voMan.addIdentity(idA[1]);
		voMan.addIdentity(idA[2]);
		voMan.addIdentity(idA[5]);
		voMan.addIdentity(idA[3], idA[0]);
		voMan.addIdentity(idA[4], idA[3]);
		voMan.addIdentity(idA[6], idA[5]);
		
		voMan.setIdentityLabel(idA[3], "changed");
		voMan.setIdentityStatus(idA[3], false);
		
		List<Identity> ids = voQuery.getAllIdentities();
		if (ids.size() != 8)
			fail("Should get 8 identities, got " + ids.size());
		for (Identity id: ids)
		{
			int i=0;
			for (; i< idA.length; i++)
				if (idA[i].equals(id))
				{
					if (comIds[i])
						fail("Identity " + id + " added twice");
					comIds[i] = true;
					if (i == 3 || i == 0 || i == 4)
						if (!id.getLabel().equals("changed"))
							fail("Wrong label after change: " + id.getLabel());
					if (i == 3)
					{
						if (!id.isInactive())
							fail("Identity inactive state was not set");
					} else
						if (id.isInactive())
							fail("Created identity is inactive while shouldn't");
					break;
				}
			if (i == idA.length && !id.getLabel().equals("VOADMIN"))
				fail("Identity " + id + " not added");
		}
		
		for (Identity id: idA)
			voMan.removeIdentity(id);
		ids = voQuery.getAllIdentities();
		if (ids.size() != 1)
			fail("Should get 1 identity, got " + ids.size());
		voMan.purgeHistoricalData(new Date());
	}
}














